@@ -47,16 +47,14 @@ typedef OSStatus (*CallBackType)(int);
   NSMutableArray *headArray;
   NSMutableArray *candArray;
 
-  int nrCandidates;
+  NSSize origSize;
+  int lineHeight;
+
   int candIndex;
-  int layoutBegin;
-  int isActive;
-  int displayLimit;
-  int selectCount;
 }
 
 - (void)setCallBack:(CallBackType)callBack;
-- (void)showWindow:(int)qdX:(int)qdY;
+- (void)showWindow:(int)qdX:(int)qdY:(int)height;
 - (void)hideWindow;
 - (void)reloadData;
 - (UniCharPtr)getCandidate:(int)index;
@@ -66,5 +64,8 @@ typedef OSStatus (*CallBackType)(int);
 - (void)selectCandidate:(int)index;
 - (void)deselectCandidate;
 - (void)setPage:(int)index:(int)max;
+- (void)replaceWindow:(int)replyX:(int)replyY;
+
 + (id)sharedController;
+
 @end
