@@ -96,7 +96,11 @@ MUIMSessionHandle *gSessionList = NULL;
 
 uim_context gUC = NULL;
 
+#ifdef NEW_HELPER
+Boolean gUimIsHelperConnect;
+#else
 int gUimFD;
+#endif
 CFSocketRef gUimSock;
 CFRunLoopSourceRef gUimRun;
 CFSocketContext gSockContext;
@@ -274,7 +278,11 @@ MUIMInitialize(ComponentInstance inComponentInstance,
 
   uim_init();
 
+#ifdef NEW_HELPER
+  gUimIsHelperConnect = FALSE;
+#else
   gUimFD = -1;
+#endif
   gUimSock = NULL;
   gUimRun = NULL;
 
@@ -358,7 +366,6 @@ MUIMSessionOpen(ComponentInstance inComponentInstance,
     (**outSessionHandle)->fOldPreeditLen = 0;
 
     (**outSessionHandle)->fBundleRef = NULL;
-
     (**outSessionHandle)->fWindowOpened = false;
   }
   else
@@ -445,9 +452,8 @@ MUIMSessionDeactivate(MUIMSessionHandle inSessionHandle)
               gActiveSession);
 #endif
 
-#if ENABLE_HELPER
-  uim_helper_client_focus_out(gUC);
-#endif
+  // commit preedit string (multi context)
+  MUIMSessionFix(inSessionHandle);
 
   /*
   if (inSessionHandle == gActiveSession) {
@@ -515,6 +521,41 @@ MUIMSessionEvent(MUIMSessionHandle inSessionHandle, EventRef inEventRef)
   return handled;
 }
 
+ComponentResult
+MUIMSessionFix(MUIMSessionHandle inSessionHandle)
+{
+  UniCharPtr str = NULL;
+  UInt32 len = 0, cursorPos = 0;
+
+#if DEBUG_PREEDIT
+  DEBUG_PRINT("MUIMSessionFix() fFixLen=%lu\n", (*inSessionHandle)->fFixLen);
+#endif
+
+  if ((*inSessionHandle)->fFixLen == 0) {
+    // commit preedit string (single context)
+    GetPreeditString(inSessionHandle, &str, &len, &cursorPos);
+    if (len > 0) {
+      (*inSessionHandle)->fFixBuffer = str;
+      (*inSessionHandle)->fFixLen = len;
+
+      MUIMUpdateActiveInputArea(inSessionHandle, TRUE);
+
+      free((*inSessionHandle)->fFixBuffer);
+      (*inSessionHandle)->fFixBuffer = NULL;
+      (*inSessionHandle)->fFixLen = 0;
+
+      uim_reset_context(gUC);
+    }
+
+#if ENABLE_HELPER
+    if ((*inSessionHandle)->fWindowOpened)
+      HideCandidateWindow(inSessionHandle);
+#endif
+  }
+
+  return noErr;
+}
+
 #pragma mark -
 
 MUIMSessionHandle
@@ -556,9 +597,22 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
     isScriptKey = TRUE;
   
   /* Then convert normal keys */
-  if (key == 0)
+  if (key == 0) {
     key = ConvertKeyVal(inCharCode, inModifiers);
 
+    // convert control sequence to normal charactor
+    // (when <control> + <special charactor>)
+    if (inModifiers & controlKey) {
+      for (i = 0; CharToKey[i].ckey; i++) {
+        if (CharToKey[i].charcode == inCharCode) {
+          key = CharToKey[i].ckey;
+          break;
+        }
+      }
+    }
+  }
+
+
   mod = ConvertModifier(inModifiers);
 
 #if DEBUG_KEYEVENT
@@ -566,9 +620,7 @@ MUIMHandleInput(MUIMSessionHandle inSessionHandle, UInt32 inKeycode,
 #endif
 
   rv = uim_press_key(gUC, key, mod);
-
   uim_release_key(gUC, key, mod);
-
   if (!rv || isScriptKey)
     handled = TRUE;
 
