@@ -47,7 +47,11 @@ extern MUIMSessionHandle *gSessionList;
 
 extern uim_context gUC;
 
+#ifdef NEW_HELPER
+extern int gUimIsHelperConnect;
+#else
 extern int gUimFD;
+#endif
 extern CFSocketRef gUimSock;
 extern CFRunLoopSourceRef gUimRun;
 extern CFSocketContext gSockContext;
@@ -68,10 +72,6 @@ ParseHelperString(const char *str);
 void
 UIMCommitString(void *ptr, const char *str)
 {
-#if DEBUG_PREEDIT
-  DEBUG_PRINT("UIMCommitString() len=%d\n", strlen(str));
-#endif
-
   //MUIMSessionHandle handle = (MUIMSessionHandle) ptr;
   MUIMSessionHandle handle = gActiveSession;
   CFMutableStringRef cf_string;
@@ -86,6 +86,11 @@ UIMCommitString(void *ptr, const char *str)
                         (*handle)->fFixBuffer);
   CFRelease(cf_string);
 
+#if DEBUG_PREEDIT
+  DEBUG_PRINT("UIMCommitString() len=%d fFixLen=%lu\n",
+              strlen(str), (*handle)->fFixLen);
+#endif
+
   if ((*handle)->fFixLen > 0) {
     MUIMUpdateActiveInputArea(handle, TRUE);
     free((*handle)->fFixBuffer);
@@ -269,11 +274,20 @@ UIMUpdatePropList(void *inPtr, const char *inStr)
                         (strlen(kPropListUpdate) + 1 +
                          strlen(inStr) + 1));
   if (tmp) {
-    if (gUimFD >= 0) {
+#ifdef NEW_HELPER
+    if (gUimIsHelperConnect)
+#else
+    if (gUimFD >= 0)
+#endif
+    {
       snprintf(tmp, strlen(kPropListUpdate) + 1 + strlen(inStr) + 1,
                "%s\n%s",
                kPropListUpdate, inStr);
+#ifdef NEW_HELPER
+      uim_helper_send_message(tmp);
+#else
       uim_helper_send_message(gUimFD, tmp);
+#endif
     }
     free(tmp);
   }
@@ -295,11 +309,20 @@ UIMUpdatePropLabel(void *inPtr, const char *inStr)
                         (strlen(kPropLabelUpdate) + 1 +
                          strlen(inStr) + 1));
   if (tmp) {
-    if (gUimFD >= 0) {
+#if NEW_HELPER
+    if (gUimIsHelperConnect)
+#else
+    if (gUimFD >= 0)
+#endif
+    {
       snprintf(tmp, strlen(kPropLabelUpdate) + 1 + strlen(inStr) + 1,
                "%s\n%s",
                kPropLabelUpdate, inStr);
+#ifdef NEW_HELPER
+      uim_helper_send_message(tmp);
+#else
       uim_helper_send_message(gUimFD, tmp);
+#endif
     }
     free(tmp);
   }
@@ -308,6 +331,46 @@ UIMUpdatePropLabel(void *inPtr, const char *inStr)
 void
 UIMCheckHelper()
 {
+#ifdef NEW_HELPER
+  if (!gUimIsHelperConnect) {
+    if (uim_helper_init_client(UIMHelperDisconnect) == 0) {
+      gUimIsHelperConnect = TRUE;
+
+#if DEBUG_HELPER
+      DEBUG_PRINT("UIMCheckHelper() gUimIsHelperConnect=%d\n", gUimIsHelperConnect);
+#endif
+
+      if (gUimIsHelperConnect) {
+        if (!gUimSock) {
+          gSockContext.version = 0;
+          //gSockContext.info = inHandle;
+          gSockContext.info = NULL;
+          gSockContext.retain = NULL;
+          gSockContext.release = NULL;
+          gSockContext.copyDescription = NULL;
+          
+          gUimSock = CFSocketCreateWithNative(kCFAllocatorDefault, uim_helper_get_client_fd(),
+                                              kCFSocketReadCallBack, HelperRead,
+                                              &gSockContext);
+          if (!gUimSock) return;
+        }
+
+        if (!gUimRun) {
+          gUimRun = CFSocketCreateRunLoopSource(kCFAllocatorDefault, gUimSock, 0);
+          if (!gUimRun) {
+            CFRelease(gUimSock);
+            gUimSock = NULL;
+            return;
+          }
+          CFRunLoopAddSource(CFRunLoopGetCurrent(), gUimRun, kCFRunLoopDefaultMode);
+#if DEBUG_HELPER
+          DEBUG_PRINT("UIMCheckHelper() CFRunLoopGetCurrent()\n");
+#endif
+        }
+      }
+    }
+  }
+#else
   if (gUimFD < 0) {
     gUimFD = uim_helper_init_client_fd(UIMHelperDisconnect);
 
@@ -344,6 +407,7 @@ UIMCheckHelper()
       }
     }
   }
+#endif
 }
 
 static void
@@ -432,7 +496,11 @@ UIMHelperDisconnect()
   gUimRun = NULL;
   gUimSock = NULL;
 
+#ifdef NEW_HELPER
+  gUimIsHelperConnect = FALSE;
+#else
   gUimFD = -1;
+#endif
 }
 
 void
@@ -442,8 +510,13 @@ UIMHelperClose()
   DEBUG_PRINT("UIMHelperClose()\n");
 #endif
 
+#ifdef NEW_HELPER
+  if (gUimIsHelperConnect)
+    uim_helper_close_client();
+#else
   if (gUimFD >= 0)
     uim_helper_close_client_fd(gUimFD);
+#endif
 }
 
 static void
